<?php

namespace Gogs\API\Request {

    /** 
     * Collection is a collection of data of one type.
     *
     * @see Users
     * @author Joachim M. Giaever (joachim[]giaever.org)
     * @version 0.1.1
     */
    abstract class Collection extends Base implements \Gogs\Lib\ArrayIterator {

        private $objs;

        public function __construct(string $api_url, string $api_token, Collection $other = null) {
            parent::__construct($api_url, $api_token);

            if ($other != null)
                $this->objs = $others->copy();
            else
                $this->objs = new \Gogs\Lib\Collection();
        }

        public function copy() {
            return new Collection($this);
        }

        /**
         * Add an object to the collection.
         *
         * When adding a key the object will be stored
         * on the particual key, also overwriting existing data.
         *
         * @param mixed $obj Element to store
         * @param mixed $key Index key to store on
         * @return mixed|int The index key. If key is null the returned value will be an integer.
         */
        public function add($obj, $key = null) {
            $this->objs->set($obj, $key);
            return $key == null ? $this->objs->len() - 1 : $key;
        }

        /** 
         * Remove an element in collection.
         *
         * The function will first look for the element as a
         * index key, but if its not found it will look for the
         * element as a value.
         *
         * Deep functions only when the value is given and not the key.
         *
         * @deprecated 0.1.1 Will be removed in future release
         * @param mixed $any Index key or element value
         * @param bool $deep Delete every item and not just the first 
         * @return bool
         */
        protected function remove($any, bool $deep = true) {
            return $objs->remove($any, $deep);
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function all() {
            return $this->objs->copy()->all();
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function len() {
            return $this->objs->len();
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function by_key($idx) {
            return $this->objs->by_key($idx);
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function next() {
            return $this->objs->next();
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function prev() {
            return $this->objs->prev();
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function current() {
            return $this->objs->current();
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function reset() {
            return $this->objs->reset();
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function sort(callable $f) {
            return $this->objs->copy()->sort($f);
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function limit(int $lim) {
            return $this->objs->copy()->limit($lim);
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function offset(int $off) {
            return $this->objs->copy()->offset($off);
        }

        /**
         * @see \Gogs\Lib\ArrayIterator
         */
        public function reverse() {
            return $this->objs->copy()->reverse();
        }

        /** 
         * Search for an object.
         *
         * @param array $params Parameters
         * @return \Gogs\Lib\Collection
         */
        abstract public function search(array $params = array());

        /**
         * Sort the object
         *
         * Should call sort on parent with the specified sort method,
         * given by $flag
         *
         * @param int $flag Sorting flag
         * @return \Gogs\Lib\Collection
         */
        abstract public function sort_by(int $flag = \Gogs\Lib\ArrayIterator::SORT_INDEX);
    }

}
?>
